from .base_api import BaseApi
import json


class MouserApi:
    def __init__(self):
        self.api = BaseApi(api_settings_key='mouser_api_key',
                           root_url='https://api.mouser.com/api/v1',
                           api_key_query='apiKey')

    @staticmethod
    def parse_and_check_for_errors(content):
        data = json.loads(content)
        errors = data['Errors']
        if len(errors) > 0:
            raise "Error(s): {}".format(errors)
        return data

    def search_keyword(self, keyword):
        content = self.api.request('/search/keyword', data={
            "SearchByKeywordRequest": {
                "keyword": keyword,
                "records": 0,
                "startingRecord": 0,
                "searchOptions": "",
                "searchWithYourSignUpLanguage": ""
            }
        })
        data = self.parse_and_check_for_errors(content)
        return data["SearchResults"]

    def get_manufacturer_list(self):
        content = self.api.request('/search/manufacturerlist')
        data = self.parse_and_check_for_errors(content)
        return data["MouserManufacturerList"]

    def search_part(self, part_number):
        content = self.api.request('/search/partnumber', data={
            "SearchByPartRequest": {
                "mouserPartNumber": part_number,
                "partSearchOptions": "",
            }
        })
        data = self.parse_and_check_for_errors(content)
        return data["SearchResults"]

    def search_part_and_manufacturer(self, part_number, manufacturer_id):
        content = self.api.request('/search/partnumberandmanufacturer', data={
            "SearchByPartMfrRequest": {
                "manufacturerId": manufacturer_id,
                "mouserPartNumber": part_number,
                "partSearchOptions": "",
            }
        })
        data = self.parse_and_check_for_errors(content)
        return data["SearchResults"]


class Mouser:
    def __init__(self):
        self.api = MouserApi()

    def search_and_match(self, manufacturer_part_number, quantity=1):
        # manufacturer_list = self.api.get_manufacturer_list()
        # TODO: possibly get manufacturer id from manufacturer list, do a fuzzy lookup using manufacturer name
        #  to reduce results
        results = self.api.search_part(part_number=manufacturer_part_number)
        seller_parts = []
        optimal_part = None
        for part in results['Parts']:
            seller_part = {
                'part_number': part['ManufacturerPartNumber'],
                'manufacturer': part['Manufacturer'],
                'description': part['Description'],
                'data_sheet': part['DataSheetUrl'],
                'stock': part['Availability'],
                'lead_time': part['LeadTime'],
                'prices': [],
            }

            for pb in part['PriceBreaks']:
                moq = int(pb['Quantity'])
                price = float(pb['Price'].strip('$'))
                currency = pb['Currency']
                order_quantity = quantity if quantity > moq else moq
                order_price = order_quantity * price
                price_break = {
                    'price': price,
                    'order_quantity': order_quantity,
                    'order_price': order_price,
                    'moq': moq,
                }
                if optimal_part is None or (order_price < optimal_part['order_price'] and currency == 'USD'):
                    optimal_part = seller_part.copy()
                    optimal_part.update(price_break)
                seller_part['prices'].append(price_break)
            seller_parts.append(seller_part)

        match = {
            'seller_parts': seller_parts,
            'optimal_seller_part': optimal_part,
        }
        return match



# This file is to have no project dependencies


def increment_char(c):
    """
    Increment an uppercase character, returning 'A' if 'Z' is given
    """
    return chr(ord(c) + 1) if c != 'Z' else 'A'


def increment_str(s):
    lpart = s.rstrip('Z')
    num_replacements = len(s) - len(lpart)
    new_s = lpart[:-1] + increment_char(lpart[-1]) if lpart else 'A'
    new_s += 'A' * num_replacements
    return new_s


# The following function is based upon code from Jeff Atwood, see:
#
#       https://blog.codinghorror.com/sorting-for-humans-natural-sort-order/
#
# Code has been adapted for for use as sort function for Python sorted(). Enables sorting an 
# iterable whose items are strings represented by a mix of alphanumeric characters. For the
# default sort for {'R14', 'R5'} is:
#
#   R14 R5
#
# but with prep_for_sorting_nicely the sort will be what is more naturally expected:
#
#   R5 R14
#

import re


def prep_for_sorting_nicely(item):
    convert = lambda text: int(text) if text.isdigit() else text
    alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
    return alphanum_key(item)


# Convert a string with delimited fields into a list of fields. Delimiters are comma,
# semi-colon, colon, tab, or blank space. Fields may contain any printable character.
def listify_string(st):
    ss = re.split(' |:|;|,|\t|\n', st)
    split_st = []
    for s in ss:
        s_strip = s.strip()
        if len(s_strip) != 0:
            split_st.append(s_strip)
    return split_st


# Convert a list of items into a comma-separated string without any surrounding brackets, 
# for example:
#
# list = [1, 2, 3 4]
#
# becomes '1, 2, 3, 4'
#
# as compared to str(list) which
#
# becomes '[1, 2, 3 4]'
def stringify_list(li):
    return ', '.join(str(x) for x in li)


# Check a string reference designator for duplicates as compared to a running set of 
# reference already seen. A reference designator may contain multiple delimited references,
# so need to check the new designator for duplicates before checking against references
# already seen. All duplicate references are added to the set duplicate_refs.
def check_references_for_duplicates(new_refs, seen_refs, duplicate_refs):
    new_refs_list = listify_string(new_refs)
    new_refs_set = set()
    for r in new_refs_list:
        if r in new_refs_set:
            duplicate_refs.add(r)
        else:
            new_refs_set.add(r)
            if r in seen_refs:
                duplicate_refs.add(r)
        seen_refs.add(r)


# Given a string that represents a number, returns a string that eliminates trailing zeros
# and decimal point if any from the input. For example, 25.000 become 25. If the input
# string that does not represent a number then the original string is returned.
def strip_trailing_zeros(num):
    found = False
    for c in num:
        if c.isdigit():
            found = True
        elif c not in ['-', '+', '.']:
            found = False
            break
    return ('%.15f' % float(num)).rstrip('0').rstrip('.') if found else num


# Input a dict with a list of key options, return the value if it exists, else None
def get_from_dict(input_dict, key_options):
    for key in key_options:
        val = input_dict.get(key, None)
        if val:
            return val
    return None


from bom.models import Part, PartClass, Seller, SellerPart, Subpart, \
    Manufacturer, Organization, ManufacturerPart, PartRevision, Assembly


def create_a_fake_organization(user, free=False):
    org, created = Organization.objects.get_or_create(
        name="Atlas",
        subscription='F' if free else 'P',
        number_item_len=4,
        owner=user)
    return org


def create_some_fake_part_classes(organization):
    pc1, c = PartClass.objects.get_or_create(code=500, name='Wendy', comment='Mechanical Switches', organization=organization)
    pc2, c = PartClass.objects.get_or_create(code=200, name='Archibald', comment='', organization=organization)
    pc3, c = PartClass.objects.get_or_create(code=503, name='Ghost', comment='Like Kasper', organization=organization)
    return pc1, pc2, pc3


def create_a_fake_subpart(part_revision, reference="U1", count=4):
    sp1 = Subpart(
        part_revision=part_revision,
        reference=reference,
        count=count)
    sp1.save()

    return sp1


def create_a_fake_assembly():
    assy = Assembly.objects.create()
    return assy


def create_a_fake_assembly_with_subpart(part_revision, reference="D4", count=4):
    assy = create_a_fake_assembly()
    subpart = create_a_fake_subpart(part_revision, reference, count)
    assy.subparts.add(subpart)
    return assy


def create_a_fake_part_revision(part, assembly, description="Brown dog", revision="1"):
    pch, created = PartRevision.objects.get_or_create(part=part, revision=revision, defaults={
        'description': description,
        'revision': revision,
        'attribute': "Voltage",
        'value': "3.3",
        'assembly': assembly,
    })
    return pch


def create_some_fake_sellers(organization):
    s1, c = Seller.objects.get_or_create(name='Mouser', organization=organization)
    s2, c = Seller.objects.get_or_create(name='Digi-Key', organization=organization)
    s3, c = Seller.objects.get_or_create(name='Archibald', organization=organization)
    return s1, s2, s3


def create_some_fake_manufacturers(organization):
    m1, c = Manufacturer.objects.get_or_create(name='STMicroelectronics', organization=organization)
    m2, c = Manufacturer.objects.get_or_create(name='Nordic Semiconductor', organization=organization)
    m3, c = Manufacturer.objects.get_or_create(name='Murata', organization=organization)
    return m1, m2, m3


def create_a_fake_seller_part(
        seller,
        manufacturer_part,
        moq,
        mpq,
        unit_cost,
        lead_time_days,
        nre_cost=None):
    sp1 = SellerPart(
        seller=seller,
        manufacturer_part=manufacturer_part,
        minimum_order_quantity=moq,
        minimum_pack_quantity=mpq,
        unit_cost=unit_cost,
        lead_time_days=lead_time_days,
        nre_cost=nre_cost)
    sp1.save()

    return sp1


def create_some_fake_parts(organization):
    (pc1, pc2, pc3) = create_some_fake_part_classes(organization=organization)
    (m1, m2, m3) = create_some_fake_manufacturers(organization=organization)

    pt1 = Part(
        number_class=pc2,
        number_item='3333',
        organization=organization)
    pt1.save()
    mp1 = ManufacturerPart(part=pt1, manufacturer=m1, manufacturer_part_number='STM32F401CEU6')
    mp1.save()
    pt1.primary_manufacturer_part = mp1
    pt1.save()
    assy = create_a_fake_assembly()
    pr1 = create_a_fake_part_revision(part=pt1, assembly=None)

    pt2 = Part(
        number_class=pc1,
        organization=organization)
    pt2.save()
    mp2 = ManufacturerPart(part=pt2, manufacturer=None, manufacturer_part_number='GRM1555C1H100JA01D')
    mp2.save()
    pt2.primary_manufacturer_part = mp2
    pt2.save()
    assy2 = create_a_fake_assembly_with_subpart(part_revision=pr1)
    pr2 = create_a_fake_part_revision(part=pt2, assembly=assy2)

    pt3 = Part(number_class=pc3, organization=organization)
    pt3.save()
    mp3 = ManufacturerPart(part=pt3, manufacturer=m3, manufacturer_part_number='NRF51822')
    mp3.save()
    assy3 = create_a_fake_assembly_with_subpart(part_revision=pr2)
    subpart3 = create_a_fake_subpart(pr1, count=10, reference="")
    assy3.subparts.add(subpart3)
    assy3.save()
    create_a_fake_part_revision(part=pt3, assembly=assy3)
    create_a_fake_part_revision(part=pt3, assembly=assy3, revision="2")

    # Create a part with no PartRevision
    pt4 = Part(number_class=pc1, number_item='4444', organization=organization)
    pt4.save(no_part_revision=True)

    # Create a part with a PartRevision with no assembly - no longer happens due to PartRevision save override
    pt5 = Part(number_class=pc1, number_item='5555', organization=organization)
    pt5.save()
    create_a_fake_part_revision(pt5, None)

    (s1, s2, s3) = create_some_fake_sellers(organization=organization)

    create_a_fake_seller_part(
        s1,
        mp1,
        moq=1,
        mpq=1,
        unit_cost=0,
        lead_time_days=None,
        nre_cost=0,)
    create_a_fake_seller_part(
        s1,
        mp1,
        moq=1,
        mpq=1,
        unit_cost=1.2,
        lead_time_days=20,
        nre_cost=500)
    create_a_fake_seller_part(
        s2,
        mp1,
        moq=1000,
        mpq=5000,
        unit_cost=0.1005,
        lead_time_days=7,
        nre_cost=0,)
    create_a_fake_seller_part(
        s2,
        mp2,
        moq=200,
        mpq=200,
        unit_cost=0.5,
        lead_time_days=47,
        nre_cost=1)

    return pt1, pt2, pt3, pt4


def create_some_fake_data(user):
    o = create_a_fake_organization(user)
    return create_some_fake_parts(o)


def create_all_part_classes():
    PartClass.objects.get_or_create(code=100, name='Assembly, Top Level SKU\'s, Finished Goods',
                                    comment='Ready to ship product; Includes packout and literature')
    PartClass.objects.get_or_create(code=101, name='Assembly, Final, Elec/Mech',
                                    comment='Fully built product without Literature or packout materials')
    PartClass.objects.get_or_create(code=102, name='Assembly, Sub-Assy, Elec/Mech', comment='')
    PartClass.objects.get_or_create(code=103, name='Assembly, Cable', comment='')
    PartClass.objects.get_or_create(code=104, name='Assembly, Packaging', comment='')
    PartClass.objects.get_or_create(code=105, name='Kit, Spare/Upgrade',
                                    comment='Includes Spares, Documentation Kits, Upgrade Kits')
    PartClass.objects.get_or_create(code=106, name='Assembly, Printed Circuit Board (PBCA)', comment='')
    PartClass.objects.get_or_create(code=107, name='PCB, Fab', comment='')
    PartClass.objects.get_or_create(code=108, name='Gerber File/X-Y data/CAD data', comment='')
    PartClass.objects.get_or_create(code=109, name='Schematic/Test Diagram', comment='')
    PartClass.objects.get_or_create(code=110, name='Mechanical Reference, PCB', comment='')
    PartClass.objects.get_or_create(code=111, name='Mechanical Reference, PCBA', comment='')
    PartClass.objects.get_or_create(code=112, name='Programmed Device, Firmware/Software', comment='')
    PartClass.objects.get_or_create(code=113, name='Program File, Firmware, Software', comment='')
    PartClass.objects.get_or_create(code=114, name='Build-To-Print, Sheet Metal', comment='')
    PartClass.objects.get_or_create(code=115, name='Build-To-Print, Cast Metal', comment='')
    PartClass.objects.get_or_create(code=116, name='Build-To-Print, Extruded',
                                    comment='Applies for both plastic and metal')
    PartClass.objects.get_or_create(code=117, name='Build-To-Print, Machined',
                                    comment='Applies for both plastic and metal')
    PartClass.objects.get_or_create(code=118, name='Build-To-Print, Molded',
                                    comment='Applies for both Injection and Compression molded')
    PartClass.objects.get_or_create(code=119, name='Build-To-Print, Formed',
                                    comment='Applies for plastic only; Formed sheet metal goes under sheet metal')
    PartClass.objects.get_or_create(code=120, name='Build-To-Print, Die-Cut',
                                    comment='Applied to foams and PSAs (Pressure Sensitive Adhesives)')
    PartClass.objects.get_or_create(code=121, name='Build-To-Print, Mechanical',
                                    comment='Applies to all Build-To-Print Mech Parts that do not fall into one of the other specific groups.')
    PartClass.objects.get_or_create(code=122, name='Build-To-Print, Printed Material',
                                    comment='Includes Custom Labels, Product Literature, etc.')
    PartClass.objects.get_or_create(code=123, name='Build-To-Print, Packaging', comment='')
    PartClass.objects.get_or_create(code=124, name='Drawing',
                                    comment='For any drawing that does not use the Item PN as the dwg PN')
    PartClass.objects.get_or_create(code=125, name='Artwork',
                                    comment='A/W for printing, silkscreening of labels, sheetmetal, etc.')
    PartClass.objects.get_or_create(code=126, name='Release Notes/Protocols', comment='')
    PartClass.objects.get_or_create(code=127, name='Custom Electronic Components', comment='Example: Custom Sensors')
    PartClass.objects.get_or_create(code=200, name='Adhesive', comment='Includes Loctite, Epoxy, Tape, etc.')
    PartClass.objects.get_or_create(code=201, name='Battery/Charger', comment='')
    PartClass.objects.get_or_create(code=202, name='Cable & Wire', comment='')
    PartClass.objects.get_or_create(code=203, name='Connector, Cable/Harness', comment='')
    PartClass.objects.get_or_create(code=204, name='Connector, PC Mountable', comment='')
    PartClass.objects.get_or_create(code=205, name='Connector, Misc', comment='')
    PartClass.objects.get_or_create(code=206, name='Cable Hardware', comment='Includes Clamps, Ties, etc.')
    PartClass.objects.get_or_create(code=207, name='Capacitor', comment='')
    PartClass.objects.get_or_create(code=208, name='Circuit Breaker/Filter', comment='')
    PartClass.objects.get_or_create(code=209, name='Crystal', comment='')
    PartClass.objects.get_or_create(code=210, name='Delay Line', comment='')
    PartClass.objects.get_or_create(code=211, name='Diode', comment='')
    PartClass.objects.get_or_create(code=212, name='Fan & Fan Accessories', comment='')
    PartClass.objects.get_or_create(code=213, name='Fuse & Fuse Hardware', comment='')
    PartClass.objects.get_or_create(code=214, name='Hardware',
                                    comment='Includes Screws, Nuts, Washers, Springs, Std-offs, Inserts, Fasteners, etc')
    PartClass.objects.get_or_create(code=215, name='Heatsink', comment='')
    PartClass.objects.get_or_create(code=216, name='IC', comment='')
    PartClass.objects.get_or_create(code=217, name='Inductor', comment='')
    PartClass.objects.get_or_create(code=218, name='Insulator', comment='')
    PartClass.objects.get_or_create(code=219, name='Label', comment='')
    PartClass.objects.get_or_create(code=220, name='Led', comment='/Light')
    PartClass.objects.get_or_create(code=221, name='Packaging Material',
                                    comment='Includes Bags, Boxes, Pallets, etc. Non-Custom only.')
    PartClass.objects.get_or_create(code=222, name='Rectifier', comment='')
    PartClass.objects.get_or_create(code=223, name='Resistor', comment='')
    PartClass.objects.get_or_create(code=224, name='Resistor Network', comment='')
    PartClass.objects.get_or_create(code=225, name='Socket', comment='')
    PartClass.objects.get_or_create(code=226, name='Switch', comment='')
    PartClass.objects.get_or_create(code=227, name='Terminal', comment='Includes Ring, Spade, Butt, etc.')
    PartClass.objects.get_or_create(code=228, name='Transformer', comment='')
    PartClass.objects.get_or_create(code=229, name='Transistor', comment='')
    PartClass.objects.get_or_create(code=230, name='Tubing', comment='All types')
    PartClass.objects.get_or_create(code=231, name='Sensor', comment='')
    PartClass.objects.get_or_create(code=232, name='Power Supply', comment='')
    PartClass.objects.get_or_create(code=233, name='Enclosures', comment='Off-The-Shelf Enclosures')
    PartClass.objects.get_or_create(code=234, name='Varistor', comment='')
    PartClass.objects.get_or_create(code=235, name='Ferrites', comment='')
    PartClass.objects.get_or_create(code=236, name='Suppressor', comment='')
    PartClass.objects.get_or_create(code=237, name='Misc Material', comment='')
    PartClass.objects.get_or_create(code=238, name='Electronic Assy (Non-Custom)', comment='')
    PartClass.objects.get_or_create(code=239, name='Antenna', comment='')



# Generated by Django 2.2.2 on 2019-06-27 04:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0020_auto_20190627_0207'),
    ]

    operations = [
        migrations.AlterField(
            model_name='partrevision',
            name='attribute',
            field=models.CharField(blank=True, default=None, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='value',
            field=models.CharField(blank=True, default=None, max_length=255, null=True),
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-bom',
    version='1.101',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='A simple Django app to manage a bill of materials.',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://www.indabom.com/',
    author='Mike Kasparian',
    author_email='mpkasp@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'python-social-auth',
        'social-auth-app-django',
        'google-api-python-client',
        'django-materializecss-form',
    ],
)
